@@ -2,12 +2,12 @@ import sys
 import click
 
 from toot import api
-from toot.cli import InstanceParamType, cli, get_context, pass_context, Context
-from typing import Optional
+from toot.cli import NOTIFICATION_TYPE_CHOICES, InstanceParamType, cli, get_context, pass_context, Context, json_option
+from typing import Optional, Tuple
 from toot.cli.validators import validate_instance
 
 from toot.entities import Notification, Status, from_dict
-from toot.output import print_notifications, print_timeline
+from toot.output import print_notifications, print_timeline, print_warning
 
 
 @cli.command()
@@ -49,7 +49,7 @@ def timeline(
     once: bool,
     count: int,
 ):
-    """Show recent items in a timeline
+    """Show timelines (deprecated, use `toot timelines` instead)
 
     By default shows the home timeline.
     """
@@ -120,15 +120,31 @@ def bookmarks(
     help="Reverse the order of the shown notifications (newest on top)"
 )
 @click.option(
-    "--mentions", "-m", is_flag=True,
-    help="Show only mentions"
+    "--type", "-t", "types",
+    type=click.Choice(NOTIFICATION_TYPE_CHOICES),
+    multiple=True,
+    help="Types to include in the result, can be specified multiple times"
 )
+@click.option(
+    "--exclude-type", "-e", "exclude_types",
+    type=click.Choice(NOTIFICATION_TYPE_CHOICES),
+    multiple=True,
+    help="Types to exclude in the result, can be specified multiple times"
+)
+@click.option(
+    "--mentions", "-m", is_flag=True,
+    help="Show only mentions (same as --type mention, overrides --type, DEPRECATED)"
+)
+@json_option
 @pass_context
 def notifications(
     ctx: Context,
     clear: bool,
     reverse: bool,
-    mentions: int,
+    mentions: bool,
+    types: Tuple[str],
+    exclude_types: Tuple[str],
+    json: bool,
 ):
     """Show notifications"""
     if clear:
@@ -136,23 +152,26 @@ def notifications(
         click.secho("✓ Notifications cleared", fg="green")
         return
 
-    exclude = []
     if mentions:
-        # Filter everything except mentions
-        # https://docs.joinmastodon.org/methods/notifications/
-        exclude = ["follow", "favourite", "reblog", "poll", "follow_request"]
+        print_warning("`--mentions` option is deprecated in favour of `--type mentions`")
+        types = ("mention",)
 
-    notifications = api.get_notifications(ctx.app, ctx.user, exclude_types=exclude)
+    response = api.get_notifications(ctx.app, ctx.user, types=types, exclude_types=exclude_types)
 
-    if not notifications:
-        click.echo("You have no notifications")
+    if json:
+        if reverse:
+            print_warning("--reverse is not supported alongside --json, ignoring")
+        click.echo(response.text)
         return
 
+    notifications = [from_dict(Notification, n) for n in response.json()]
     if reverse:
         notifications = reversed(notifications)
 
-    notifications = [from_dict(Notification, n) for n in notifications]
-    print_notifications(notifications)
+    if notifications:
+        print_notifications(notifications)
+    else:
+        click.echo("You have no notifications")
 
 
 def _show_timeline(generator, reverse, once):
